/* stdlib.js
 * Learn 2 JS
 * Copyright 2018 Mark Damon Hughes
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

"use strict";

/** Shortcut for finding an element by ID. */
function $(elid) {
        return document.getElementById(elid);
}

/** Translate plain text into legal HTML. */
function htmlify(text) {
	return text.replace(/\&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\n/, "<br>");
}

/** Clears the output element. */
function outputClear() {
	$("output").innerHTML = "";
}

/** Writes msg to the output element. */
function output(msg) {
	$("output").innerHTML += msg;
}

/** Simple debugging tool, if you aren't sure what's in obj: output(htmlify(objectToString(obj))) */
function objectToString(obj) {
	let text = "";
	for (let k in obj) {
		text += k+": "+obj[k]+"\n";
	}
	return text;
}

/** Call testStart(), then a series of testAssert, testEqual, etc., then testFinish(), will print a report at the bottom of the page.
*/
function testStart() {
	if ( ! $("testconsole")) {
		document.body.innerHTML += "<pre id='testconsole' style='position: fixed; bottom: 0; background: red; color: white'></pre>";
	}
	document.assertsPassed = 0;
	document.assertsFailed = 0;
}

function testFinish() {
	if ($("testconsole")) {
		let msg = document.assertsPassed+" tests passed, "+document.assertsFailed+" tests failed";
		if (document.assertsFailed === 0) {
			$("testconsole").style.background = "green";
		}
		$("testconsole").innerHTML += htmlify(msg);
		console.log(msg);
	}
}

function testAssert(cond, msg) {
	if (cond) {
		if (document.assertsPassed !== undefined) {
			document.assertsPassed += 1;
		}
	} else {
		if (document.assertsFailed !== undefined) {
			document.assertsFailed += 1;
		}

		msg = msg || "Assertion failed";
		console.assert(msg);
		if ($("testconsole")) {
			$("testconsole").innerHTML += htmlify(msg)+"\n";
			console.error(msg);
		}
		//throw new Error(msg);
	}
}

function testEquals(expected, butgot, msg) {
	testAssert(expected === butgot, "Expected ((("+expected+"))) but got ((("+butgot+")))" + (msg ? ": "+msg : ""));
}
